import typing
import copy
from .ZoneSenderObject import ZoneSenderObject

class SomeipPackage(ZoneSenderObject):
    ''' someip 发送或接收的数据
    '''
    def __init__(
        self, 
        service_name: 'str' = '',
        instance_id: 'int' = -1,
        interface_name: 'str' = '',
        interface_type: 'str' = '',
        context: 'dict' = {}) -> None:
        #########################
        super().__init__()
        self.serviceName = service_name
        self.srcIp = ''
        self.srcPort = -1
        self.destIp = ''
        self.destPort = -1
        self.interfaceType = interface_type
        self.serviceId = -1
        self.instanceId = instance_id
        self.interfaceId = -1
        self.interfaceName = interface_name
        self.context = context
    
    def __str__(self) -> str:
        # return super().__str__()
        # return '{0} {1} {2}'.format(self.serviceName, self.interfaceName, self.context)
        return \
        '''
srcIp: {0}
srcPort: {1}
destIp: {2}
destPort: {3}
type: {4}
serviceId: {5}
serviceName: {6}
instanceId: {7}
interfaceId: {8}
interfaceName: {9}
context: {10}
        '''.format(
            self.srcIp,
            self.srcPort,
            self.destIp,
            self.destPort,
            self.interfaceType,
            self.serviceId,
            self.serviceName,
            self.instanceId,
            self.interfaceId,
            self.interfaceName,
            self.context
        )


import json
import typing

import grpc

from . import ZoneSenderData
from .ObjIo import *

from .Protos import SomeIpNode_pb2, SomeIpNode_pb2_grpc


class SomeipNodeClient(object):
    def __init__(self) -> None:
        """ SomeipNode 的客户端
        """
        self._someipNodeStub = SomeIpNode_pb2_grpc.SomeIpNodeStub(
            channel=grpc.insecure_channel(
                target='{0}:{1}'
                    .format(
                        ZoneSenderData.SOMEIP_STACK_NODE_IP, 
                        ZoneSenderData.SOMEIP_STACK_NODE_PORT),
                options = ZoneSenderData.GRPC_OPTIONS
            )
        )

    def StartSomeIpStack(self, ip_addr: 'str', iface: 'str') -> 'int':
        ''' 启动 SomeIp 协议栈

        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            res_ = self._someipNodeStub.StartSomeIpStack(
                SomeIpNode_pb2.Common__pb2.net_info(
                    ip_addr = ip_addr,
                    iface = iface
                )
            )
            print('启动Someip协议栈, result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.reason
        except Exception as e_:
            print(e_)
            return 1000

    def StopSomeIpStack(self) -> 'int':
        '''关闭 SomeIp 协议栈

        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            res_ = self._someipNodeStub.StopSomeIpStack(
                SomeIpNode_pb2.Common__pb2.empty()
            )
            print('关闭SomeIp协议栈, result: {0}, reason:{1}'.format(res_.result, res_.reason))
            return res_.reason
        except Exception as e_:
            print(e_)
            return 1000

    def AddSomeIpArxml(self, arxml_path: 'str') -> 'int':
        ''' 添加一个 SomeIp Arxml 文件, 可以重复调用添加多个

        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            res_ = self._someipNodeStub.AddSomeIpArxml(
                SomeIpNode_pb2.Common__pb2.file_path(
                    path = arxml_path
                )
            )
            print('加载 SomeIp Arxml 文件 result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.reason
        except Exception as e_:
            print(e_)
            return 1000

    def GetSomeIpServiceInfos(self, return_d: 'dict') -> 'int':
        ''' 获取当前已经加载的 SomeIp Arxml Info

        :param return_d: 如果获取成功，将会把获取的信息填充到该字典中
        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._someipNodeStub.GetSomeIpServiceInfos(
                SomeIpNode_pb2.Common__pb2.empty()
            )
            if (not res_.result.result == 0):
                return res_.result.result
            return_d.clear()
            return_d.update(json.loads(res_.json_str_info))
            return res_.result.result
        except Exception as e_:
            print(e_)
            return 1000

    def UpdateSomeIpServiceConfig(
        self, 
        service_name: 'str', 
        instance_id: 'int', 
        service_type: 'str') -> 'int':
        ''' 更新SomeIp中服务的信息

        :param service_id: str SomeIp Service ID
        :param instance_id: int SomeIp Service Instance ID
        :param service_type: str 可以是 consumer|provider 表示该服务设置为什么类型
        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._someipNodeStub.UpdateSomeipServiceConfig(
                SomeIpNode_pb2.service_tag(
                    service_name = service_name,
                    instance_id = instance_id,
                    service_type = service_type
                )
            )
            print('更新 SomeIp 服务设定 result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def Reset(self) -> 'int':
        ''' 复位 SomeIp 协议栈，并清空 SomeIp 服务配置

        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._someipNodeStub.Reset(
                SomeIpNode_pb2.Common__pb2.empty()
            )
            print('复位 SomeIp 协议栈 result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def GetSomeIpStackStatus(self) -> 'int':
        '''获取当前 Someip Stack 的状态

        :return: int\n
            - 0 正在运行\n
            - 1 协议栈未启动\n
            - 2 协议栈未初始化\n
            - 1000 error
        '''
        try:
            res_ = self._someipNodeStub.GetSomeipStackStatus(
                SomeIpNode_pb2.Common__pb2.empty()
            )
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def SomeipCallSync(
        self, 
        someip_package_in: 'SomeipPackage', 
        someip_package_out: 'SomeipPackage', 
        timeout: 'int' = 1000) -> 'int':
        ''' 同步调用 SomeipCall

        :param someip_package_in: 要调用的 SomeipPackage
        :param someip_package_out: 如果成功调用，则将返回的 SomeipPackage 填充到该对象中
        :param timeout: 超时参数，单位 ms
        :return: int\n
            - 0 正在运行\n
            - 1 超时\n
            - 1000 error
        '''
        try:
            d_ = {
                'sv_name': someip_package_in.serviceName,
                'ince_id': someip_package_in.instanceId,
                'if_name': someip_package_in.interfaceName,
                'if_type': someip_package_in.interfaceType,
                'context': someip_package_in.context
            }
            res_ = self._someipNodeStub.SomeipCallSync(
                SomeIpNode_pb2.someip_call_context(
                    timeout = timeout,
                    str_context = json.dumps(d_)
                )
            )
            result_ = res_.result.result
            if (result_ == 0):
                # 成功回复
                recv_d_ = json.loads(res_.str_context)
                someip_package_out.srcIp = recv_d_['src_ip']
                someip_package_out.srcPort = recv_d_['src_port']
                someip_package_out.destIp = recv_d_['dest_ip']
                someip_package_out.destPort = recv_d_['dest_port']
                someip_package_out.interfaceType = recv_d_['type']
                someip_package_out.serviceId = recv_d_['sv_id']
                someip_package_out.instanceId = recv_d_['ince_id']
                someip_package_out.interfaceId = recv_d_['if_id']
                someip_package_out.interfaceName = recv_d_['if_name']
                someip_package_out.context = recv_d_['context']
                return 0
            else:
                print(res_.result.reason)
                return result_
        except Exception as e_:
            print(e_)
            return 1000


import json
import os
import time
import typing
import uuid
from concurrent.futures import ThreadPoolExecutor

import grpc
import paho.mqtt.client as mqtt

from . import ZoneSenderData

from .ObjIo import *
from .ZoneSenderFramework import ZoneSenderFramework


class ZoneSenderStone(ZoneSenderFramework):
    def __init__(self) -> None:
        super().__init__()
        ########################################
        self._threadPool = ThreadPoolExecutor(2)
        self._mqttClient = mqtt.Client(
            client_id=uuid.uuid4().hex,
            transport='websockets')
        self._mqttClient.on_connect = self._OnMqttConnect
        self._mqttClient.on_message = self._OnMqttMessage
        self._mqttClient.on_disconnect = self._OnMqttDisconnect
        self._mqttClient.connect(ZoneSenderData.MQTT_BROKER_IP, ZoneSenderData.MQTT_BROKER_PORT)
        self._mqttClient.loop_start()
        
    def Subscribe(self, obj) -> None:
        ''' 订阅对象事件，只有订阅了相关对象，才能在后续的 OnXXX 回调函数中收到对应的对象接收事件

        :param obj: 需要订阅的对象，可以是如下类型\n
            - ZoneSender.ObjIo.CanMessage\n
            - ZoneSender.ObjIo.CanFrame\n
            - ZoneSender.ObjIo.CanSignal\n
            - ZoneSender.ObjIo.CanISignalIPdu\n
            - ZoneSender.ObjIo.SomeipPackage\n
        '''
        obj_ = obj
        if (isinstance(obj_, CanMessage)):
            self._mqttClient.subscribe('zonesender/canstacknode/message/{0}/{1}'.format(obj_.channel, hex(obj_.arbitration_id).lower()))
            self._mqttClient.publish(
                topic='zonesender/canstacknode/subscribe',
                payload=json.dumps({
                    'client_id': self._mqttClient._client_id.decode('utf-8'),
                    'channel': obj_.channel,
                    'arbitration_id': obj_.arbitration_id,
                }).encode('utf-8')
            )
        elif (isinstance(obj_, CanFrame)):
            self._mqttClient.subscribe('zonesender/canparsernode/frame/{0}/{1}'.format(obj_.channel, obj_.name))
            self._mqttClient.publish(
                topic='zonesender/canparsernode/subscribe',
                payload=json.dumps({
                    'client_id': self._mqttClient._client_id.decode('utf-8'),
                    'channel': obj_.channel,
                    'name': obj_.name,
                    'type': 'frame',
                }).encode('utf-8')
            )
        elif (isinstance(obj_, CanSignal)):
            self._mqttClient.subscribe('zonesender/canparsernode/signal/{0}/{1}'.format(obj_.channel, obj_.name))
            self._mqttClient.publish(
                topic='zonesender/canparsernode/subscribe',
                payload=json.dumps({
                    'client_id': self._mqttClient._client_id.decode('utf-8'),
                    'channel': obj_.channel,
                    'name': obj_.name,
                    'type': 'signal',
                }).encode('utf-8')
            )
        elif (isinstance(obj_, CanISignalIPdu)):
            self._mqttClient.subscribe('zonesender/canparsernode/pdu/{0}/{1}'.format(obj_.channel, obj_.name))
            self._mqttClient.publish(
                topic='zonesender/canparsernode/subscribe',
                payload=json.dumps({
                    'client_id': self._mqttClient._client_id.decode('utf-8'),
                    'channel': obj_.channel,
                    'name': obj_.name,
                    'type': 'pdu',
                }).encode('utf-8')
            )
        elif (isinstance(obj_, SomeipPackage)):
            if (obj_.serviceName == ''):
                print('不能订阅空的 someip service')
                return
            self._mqttClient.subscribe('zonesender/someipnode/someippackage/{0}'.format(obj_.serviceName))
            self._mqttClient.publish(
                topic='zonesender/someipnode/subscribe',
                payload=json.dumps({
                    'client_id': self._mqttClient._client_id.decode('utf-8'),
                    'service_name': obj_.serviceName
                }).encode('utf-8')
            )
    
    def UnSubscribe(self, obj) -> None:
        ''' 取消订阅事件，调用该函数后，将不会再 OnXXX 的回调函数中收到相关消息

        :param obj: 需要取消订阅的对象，可以为如下类型\n
            - ZoneSender.ObjIo.CanMessage\n
            - ZoneSender.ObjIo.CanFrame\n
            - ZoneSender.ObjIo.CanSignal\n
            - ZoneSender.ObjIo.CanISignalIPdu\n
            - ZoneSender.ObjIo.SomeipPackage\n
        '''
        obj_ = obj
        if (isinstance(obj_, CanMessage)):
            self._mqttClient.unsubscribe('zonesender/canstacknode/message/{0}/{1}'.format(obj_.channel, hex(obj_.arbitration_id).lower()))
            self._mqttClient.publish(
                topic='zonesender/canstacknode/unsubscribe',
                payload=json.dumps({
                    'client_id': self._mqttClient._client_id.decode('utf-8'),
                    'channel': obj_.channel,
                    'arbitration_id': obj_.arbitration_id,
                }).encode('utf-8')
            )
        elif (isinstance(obj_, CanFrame)):
            self._mqttClient.unsubscribe('zonesender/canparsernode/frame/{0}/{1}'.format(obj_.channel, obj_.name))
            self._mqttClient.publish(
                topic='zonesender/canparsernode/unsubscribe',
                payload=json.dumps({
                    'client_id': self._mqttClient._client_id.decode('utf-8'),
                    'channel': obj_.channel,
                    'name': obj_.name,
                    'type': 'frame',
                }).encode('utf-8')
            )
        elif (isinstance(obj_, CanISignalIPdu)):
            self._mqttClient.unsubscribe('zonesender/canparsernode/pdu/{0}/{1}'.format(obj_.channel, obj_.name))
            self._mqttClient.publish(
                topic='zonesender/canparsernode/unsubscribe',
                payload=json.dumps({
                    'client_id': self._mqttClient._client_id.decode('utf-8'),
                    'channel': obj_.channel,
                    'name': obj_.name,
                    'type': 'pdu',
                }).encode('utf-8')
            )
        elif (isinstance(obj_, CanSignal)):
            self._mqttClient.unsubscribe('zonesender/canparsernode/signal/{0}/{1}'.format(obj_.channel, obj_.name))
            self._mqttClient.publish(
                topic='zonesender/canparsernode/unsubscribe',
                payload=json.dumps({
                    'client_id': self._mqttClient._client_id.decode('utf-8'),
                    'channel': obj_.channel,
                    'name': obj_.name,
                    'type': 'signal',
                }).encode('utf-8')
            )
        elif (isinstance(obj_, SomeipPackage)):
            self._mqttClient.unsubscribe('zonesender/someipnode/someippackage/{0}'.format(obj_.serviceName))
            self._mqttClient.publish(
                topic='zonesender/someipnode/unsubscribe',
                payload=json.dumps({
                    'client_id': self._mqttClient._client_id.decode('utf-8'),
                    'service_name':obj_.serviceName
                }).encode('utf-8')
            )

    def OnCanFrame(self, timestamp: 'float', can_frame: 'CanFrame') -> None:
        ''' [需要用户重写实现定制功能]\n
        当收到一个 CAN 报文应该做什么，此函数在收到任何订阅过的 CAN 报文的时候触发

        :param timestamp: 收到该数据的时间戳
        :param can_frame: ZoneSender.ObjIo.CanFrame 收到的 CANFrame 对象
        '''
        pass

    def OnCanPdu(self, timestamp: 'float', can_pdu: 'CanISignalIPdu') -> None:
        ''' [需要用户重写实现定制功能]\n
        当收到一个 CAN PDU 应该做什么，此函数在收到任何订阅过的 CAN PDU 的时候触发

        :param timestamp: 收到该数据的时间戳
        :param can_pdu: ZoneSender.ObjIo.CanISignalIPdu 收到的 CanISignalIPdu 对象
        '''
        pass

    def OnCanSignal(self, timestamp: 'float', can_signal: 'CanSignal') -> None:
        ''' [需要用户重写实现定制功能]\n
        当收到一个 CAN Signal 应该做什么，此函数在收到任何订阅过的 CAN Signal 的时候触发

        :param timestamp: 收到该数据的时间戳
        :param can_signal: ZoneSender.ObjIo.CanSignal 收到的 CanSignal 对象
        '''
        pass

    def OnCanMessage(self, timestamp: 'float', can_message: 'CanMessage') -> None:
        ''' [需要用户重写实现定制功能]\n
        当收到一个 CAN Message 应该做什么，此函数在收到任何订阅过的 CAN Message 的时候触发

        :param timestamp: 收到该数据的时间戳
        :param can_message: ZoneSender.ObjIo.CanMessage 收到的 CanMessage 对象
        '''
        pass

    def OnSomeipPackage(self, timestamp: 'float', someip_package: 'SomeipPackage') -> None:
        ''' [需要用户重写实现定制功能]\n
        当收到一个 SomeipPackage 应该做什么，此函数在收到任何订阅过的 SomeipPackage 的时候触发

        :param timestamp: 收到该数据的时间戳
        :param someip_package: ZoneSender.ObjIo.SomeipPackage 收到的 SomeipPackage 对象
        '''
        pass

    def SendCan(self, obj, **kwargs) -> None:
        '''发送一条CAN报文, 数据将立刻发送

        :param obj: 要发送的 CAN 对象，可以是如下类型\n
            - ZoneSender.ObjIo.CanMessage\n
            - ZoneSender.ObjIo.CanISignalIPdu\n
        :param kwargs: 附加 key:value 型参数，有如下可能:\n
            - by: str 通过什么参数来发送，有如下可能:\n
                - 'data': 通过 obj.data 的数据来发送\n
                - 'context': 通过 obj.context 的数据来发送\n
        '''
        obj_ = obj
        try:
            if (isinstance(obj_, CanMessage)):
                d_ = {
                    'id': obj_.arbitration_id,
                    'ext': obj_.is_extended_id,
                    'rem': obj_.is_remote_frame,
                    'chl': obj_.channel,
                    'dlc': obj_.dlc,
                    'd': obj_.data,
                    'fd': obj_.is_fd,
                }
                self._mqttClient.publish(
                    topic='zonesender/canstacknode/requests/send_can_message',
                    payload=json.dumps(d_).encode('utf-8')
                )
            elif (isinstance(obj_, CanISignalIPdu)):
                d_ = {
                    'name': obj_.name,
                    'chl': obj_.channel,
                    'context': obj_.context,
                    'd': obj_.data,
                }
                d_['kwargs'] = kwargs
                self._mqttClient.publish(
                    topic='zonesender/canparsernode/requests/send_can_pdu',
                    payload=json.dumps(d_).encode('utf-8')
                )
        except Exception as e_:
            print(e_)

    def SetCycleSendTask(self, period_ms: 'int', obj, times: 'int' = -1) -> None:
        ''' 设置定时发送任务，定时把 obj 发送出去

        :param period_ms: 发送周期
        :param obj: 要发送的对象，支持以下类型\n
            - ZoneSender.ObjIo.CanMessage\n
            - ZoneSender.ObjIo.CanISignalIPdu\n
            - ZoneSender.ObjIo.CanFrame\n
        :param times: 发送的次数，规则如下\n
            - \-1(默认) 一直发送\n
            - 0 停止发送，可以对已经发送的对象设置0来停止发送\n
            - 任意正整数 N，连续发送 N 次\n
        :return: None
        '''
        period_ms_ = period_ms
        obj_ = obj
        times_ = times
        try:
            if (isinstance(obj_, CanMessage)):
                # 设置循环发送 CanMessage
                d_ = {
                    'name': obj_.name,
                    'id': obj_.arbitration_id,
                    'ext': obj_.is_extended_id,
                    'rem': obj_.is_remote_frame,
                    'chl': obj_.channel,
                    'dlc': obj_.dlc,
                    'd': obj_.data,
                    'fd': obj_.is_fd,
                    'times': times_,
                    'period': period_ms_,
                }
                self._mqttClient.publish(
                    topic='zonesender/canstacknode/requests/send_can_message_cyc',
                    payload=json.dumps(d_).encode('utf-8')
                )
            elif (isinstance(obj_, CanFrame)):
                # 设置循环发送 CanFrame
                d_ = {
                    'name': obj_.name,
                    'chl': obj_.channel,
                    'times': times_,
                    'period': period_ms_,
                    'd': [],    # TODO 暂时保留 d 来发送空数据，保持接口一致
                    'context': obj_.context,
                }
                self._mqttClient.publish(
                    topic='zonesender/canparsernode/requests/send_can_frame_cyc',
                    payload=json.dumps(d_).encode('utf-8')
                )
            elif (isinstance(obj_, CanISignalIPdu)):
                # 设置循环发送 CAN PDU
                d_ = {
                    'name': obj_.name,
                    'chl': obj_.channel,
                    'context': obj_.context,
                    'times': times_,
                    'period': period_ms_,
                }
                self._mqttClient.publish(
                    topic='zonesender/canparsernode/requests/send_can_pdu_cyc',
                    payload=json.dumps(d_).encode('utf-8')
                )
        except Exception as e_:
            print(e_)

    def SomeipCallAsync(self, someip_package: 'SomeipPackage') -> None:
        ''' 作为 Client调用，请求一个 someip method | get | set

        :param someip_package: 要发送的 SomeipPackage
        :return: None
        '''
        try:
            d_ = {
                'sv_name': someip_package.serviceName,
                'ince_id': someip_package.instanceId,
                'if_name': someip_package.interfaceName,
                'if_type': someip_package.interfaceType,
                'context': someip_package.context
            }
            self._mqttClient.publish(
                topic='zonesender/someipnode/request/call',
                payload=json.dumps(d_).encode('utf-8')
            )
        except Exception as e_:
            print(e_)

    def SomeipSetDefaultAnswer(self, someip_package: 'SomeipPackage') -> None:
        ''' 作为 Server 调用，设置后台 SomeIpServer 的数据

        :param someip_package: 要设置的 SomeipPackage
        :return: None
        '''
        try:
            d_ = {
                'sv_name': someip_package.serviceName,
                'ince_id': someip_package.instanceId,
                'if_name': someip_package.interfaceName,
                'if_type': someip_package.interfaceType,
                'context': someip_package.context
            }
            self._mqttClient.publish(
                topic='zonesender/someipnode/request/setvalue',
                payload=json.dumps(d_).encode('utf-8')
            )
        except Exception as e_:
            print(e_)

    def SomeipPublish(self, someip_package: 'SomeipPackage') -> None:
        ''' 作为 Server 调用，发送一次 Notification 或者 Event

        :param someip_package: 要发布的 SomeipPackage
        :return: None
        '''
        try:
            d_ = {
                'sv_name': someip_package.serviceName,
                'ince_id': someip_package.instanceId,
                'if_name': someip_package.interfaceName,
                'if_type': someip_package.interfaceType,
                'context': someip_package.context
            }
            self._mqttClient.publish(
                topic='zonesender/someipnode/request/publish',
                payload=json.dumps(d_).encode('utf-8')
            )
        except Exception as e_:
            print(e_)

    def _OnMqttMessage(self, client: mqtt.Client, userdata, msg: mqtt.MQTTMessage) -> None:
        '''当收到MQTTMwssage的时候做什么

        :param client: mqtt client
        :param userdata: None
        :param msg: mqtt.MQTTMessage
        '''
        topic_split_ = msg.topic.split('/')
        # print(msg.topic)
        # 消息分发
        if (topic_split_[1] == 'canstacknode'):
            recv_d_ = json.loads(msg.payload.decode('utf-8'))
            # 收到 CAN 数据
            if (topic_split_[2] == 'message'):
                # 收到 CANMessage
                self._DealWithCanMessage(recv_d_)
        elif (topic_split_[1] == 'canparsernode'):
            recv_d_ = json.loads(msg.payload.decode('utf-8'))
            # print(recv_d_)
            if (topic_split_[2] == 'frame'):
                # 收到 CanFrame
                self._DealWithCanFrame(recv_d_)
            elif (topic_split_[2] == 'signal'):
                # 收到CANSIgnal
                self._DealWithCanSignal(recv_d_)
            elif (topic_split_[2] == 'pdu'):
                # 收到CANPdu
                self._DealWithCanISignalIPdu(recv_d_)
        elif (topic_split_[1] == 'out'):
            print(msg.payload.decode('utf-8'))
        elif (topic_split_[1] == 'someipnode'):
            recv_d_ = json.loads(msg.payload.decode('utf-8'))
            if (topic_split_[2] == 'someippackage'):
                # 收到 SomeipPackage
                # print(recv_d_)
                self._DealWithSomeipPackage(recv_d_)

    def _OnMqttConnect(self, client: mqtt.Client, userdata, flags, rc) -> None:
        '''
        连接到 MQTT Broker 的时候做什么
        '''
        self._mqttClient.subscribe('zonesender/out/info')
        self._mqttClient.subscribe('zonesender/out/warn')
        self._mqttClient.subscribe('zonesender/out/error')
        self._mqttClient.subscribe('zonesender/out/debug')

    def _OnMqttDisconnect(self, client: mqtt.Client, userdata, rc) -> None:
        '''
        当断开与MQTT Broker 连接的时候做什么
        '''
        while (True):
            try:
                client.reconnect()
                break
            except Exception as e_:
                print(e_)
            time.sleep(2)

    def _DealWithCanMessage(self, recv_d: 'dict') -> None:
        '''
        处理接收到的 CaneMessage 消息
        '''
        can_msg_ = CanMessage(
            arbitration_id=recv_d['id'], 
            channel=recv_d['chl'], 
            dlc=recv_d['dlc'], 
            data=recv_d['d'],
            is_fd=recv_d['fd'],
            is_extended_id=recv_d['ext'],
            is_remote_frame=recv_d['rem'])
        self._threadPool.submit(
            self.OnCanMessage,
            recv_d['t'], can_msg_
        )

    def _DealWithCanFrame(self, recv_d: 'dict') -> None:
        '''
        处理收到的 CanFrame 消息
        '''
        can_frame_ = CanFrame(
            name=recv_d['name'],
            channel=recv_d['chl']
        )
        can_frame_.data = recv_d['d']
        can_frame_.dlc = recv_d['dlc']
        can_frame_.id = recv_d['id']
        self._threadPool.submit(
            self.OnCanFrame,
            recv_d['t'], can_frame_
        )

    def _DealWithCanISignalIPdu(self, recv_d: 'dict') -> None:
        '''
        处理收到的 CanISignalIPdu 消息
        '''
        can_pdu_ = CanISignalIPdu(
            name=recv_d['name'],
            channel=recv_d['chl']
        )
        can_pdu_.data = recv_d['d']
        self._threadPool.submit(
            self.OnCanPdu,
            recv_d['t'], can_pdu_
        )

    def _DealWithCanSignal(self, recv_d: 'dict') -> None:
        '''
        处理收到的 CanSignal 消息
        '''
        can_signal_ = CanSignal(
            name=recv_d['name'],
            channel=recv_d['chl'],
            data=recv_d['d'],
            unit=recv_d['u'],
            mean=recv_d['m'],
        )
        self._threadPool.submit(
            self.OnCanSignal,
            recv_d['t'], can_signal_
        )

    def _DealWithSomeipPackage(self, recv_d: 'dict') -> None:
        '''
        处理接收到的 SomeipPackage
        '''
        # print(recv_d)
        someip_package_ = SomeipPackage(recv_d['sv_name'])
        someip_package_.srcIp = recv_d['src_ip']
        someip_package_.srcPort = recv_d['src_port']
        someip_package_.destIp = recv_d['dest_ip']
        someip_package_.destPort = recv_d['dest_port']
        someip_package_.interfaceType = recv_d['type']
        someip_package_.serviceId = recv_d['sv_id']
        someip_package_.instanceId = recv_d['ince_id']
        someip_package_.interfaceId = recv_d['if_id']
        someip_package_.interfaceName = recv_d['if_name']
        someip_package_.context = recv_d['context']
        self._threadPool.submit(
            self.OnSomeipPackage,
            recv_d['t'], someip_package_
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import Common_pb2 as Common__pb2
from . import SomeIpNode_pb2 as SomeIpNode__pb2


class SomeIpNodeStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.StartSomeIpStack = channel.unary_unary(
                '/SomeIpNode.SomeIpNode/StartSomeIpStack',
                request_serializer=Common__pb2.net_info.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.StopSomeIpStack = channel.unary_unary(
                '/SomeIpNode.SomeIpNode/StopSomeIpStack',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.AddSomeIpArxml = channel.unary_unary(
                '/SomeIpNode.SomeIpNode/AddSomeIpArxml',
                request_serializer=Common__pb2.file_path.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.GetSomeIpServiceInfos = channel.unary_unary(
                '/SomeIpNode.SomeIpNode/GetSomeIpServiceInfos',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=SomeIpNode__pb2.someip_info.FromString,
                )
        self.UpdateSomeipServiceConfig = channel.unary_unary(
                '/SomeIpNode.SomeIpNode/UpdateSomeipServiceConfig',
                request_serializer=SomeIpNode__pb2.service_tag.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.Reset = channel.unary_unary(
                '/SomeIpNode.SomeIpNode/Reset',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.SomeipCallSync = channel.unary_unary(
                '/SomeIpNode.SomeIpNode/SomeipCallSync',
                request_serializer=SomeIpNode__pb2.someip_call_context.SerializeToString,
                response_deserializer=SomeIpNode__pb2.someip_response_context.FromString,
                )
        self.GetSomeipStackStatus = channel.unary_unary(
                '/SomeIpNode.SomeIpNode/GetSomeipStackStatus',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )


class SomeIpNodeServicer(object):
    """Missing associated documentation comment in .proto file."""

    def StartSomeIpStack(self, request, context):
        """开启 SomeIp 协议栈
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopSomeIpStack(self, request, context):
        """停止 SomeIp 协议栈
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddSomeIpArxml(self, request, context):
        """添加 SomeIpArxml
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSomeIpServiceInfos(self, request, context):
        """获取所有的 Service 信息
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateSomeipServiceConfig(self, request, context):
        """更新 Someip service 的配置信息
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Reset(self, request, context):
        """复位
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SomeipCallSync(self, request, context):
        """同步发送Someip Call
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSomeipStackStatus(self, request, context):
        """获取 Someip 协议栈状态
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SomeIpNodeServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'StartSomeIpStack': grpc.unary_unary_rpc_method_handler(
                    servicer.StartSomeIpStack,
                    request_deserializer=Common__pb2.net_info.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'StopSomeIpStack': grpc.unary_unary_rpc_method_handler(
                    servicer.StopSomeIpStack,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'AddSomeIpArxml': grpc.unary_unary_rpc_method_handler(
                    servicer.AddSomeIpArxml,
                    request_deserializer=Common__pb2.file_path.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'GetSomeIpServiceInfos': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSomeIpServiceInfos,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=SomeIpNode__pb2.someip_info.SerializeToString,
            ),
            'UpdateSomeipServiceConfig': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateSomeipServiceConfig,
                    request_deserializer=SomeIpNode__pb2.service_tag.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'Reset': grpc.unary_unary_rpc_method_handler(
                    servicer.Reset,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'SomeipCallSync': grpc.unary_unary_rpc_method_handler(
                    servicer.SomeipCallSync,
                    request_deserializer=SomeIpNode__pb2.someip_call_context.FromString,
                    response_serializer=SomeIpNode__pb2.someip_response_context.SerializeToString,
            ),
            'GetSomeipStackStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSomeipStackStatus,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'SomeIpNode.SomeIpNode', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class SomeIpNode(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def StartSomeIpStack(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SomeIpNode.SomeIpNode/StartSomeIpStack',
            Common__pb2.net_info.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopSomeIpStack(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SomeIpNode.SomeIpNode/StopSomeIpStack',
            Common__pb2.empty.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddSomeIpArxml(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SomeIpNode.SomeIpNode/AddSomeIpArxml',
            Common__pb2.file_path.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSomeIpServiceInfos(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SomeIpNode.SomeIpNode/GetSomeIpServiceInfos',
            Common__pb2.empty.SerializeToString,
            SomeIpNode__pb2.someip_info.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateSomeipServiceConfig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SomeIpNode.SomeIpNode/UpdateSomeipServiceConfig',
            SomeIpNode__pb2.service_tag.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Reset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SomeIpNode.SomeIpNode/Reset',
            Common__pb2.empty.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SomeipCallSync(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SomeIpNode.SomeIpNode/SomeipCallSync',
            SomeIpNode__pb2.someip_call_context.SerializeToString,
            SomeIpNode__pb2.someip_response_context.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSomeipStackStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SomeIpNode.SomeIpNode/GetSomeipStackStatus',
            Common__pb2.empty.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import json
import os
import typing

from .CanParserNodeClient import CanParserNodeClient
from .CanStackNodeClient import CanStackNodeClient
from .SomeipNodeClient import SomeipNodeClient


class ZoneSenderFramework(object):
    '''
    ZoneSender 的基础类，可以用来实现对硬件的简单控制\n
    该对象有几个成员变量，可以查看相应的对象介绍页面来了解函数列表:\n
        - CanStack: ZoneSender.CanStackNodeClient\n
        - CanParser: ZoneSender.CanParserNodeClient\n
        - SomeipStack: ZoneSender.SomeipNodeClient\n
    通过调用几个成员变量的函数实现各种控制功能\n
    '''
    def __init__(self) -> None:
        super().__init__()
        self.CanStack = CanStackNodeClient()
        self.CanParser = CanParserNodeClient()
        self.SomeipStack = SomeipNodeClient()

    def Reset(self) -> None:
        ''' 复位所有功能，包括:\n
            - 复位 CAN 协议栈\n
            - 清空 CAN DB 文件\n
            - 清除所有数据记录任务\n
            - 清空 SomeipStack \n
            - 清空 SomeipStack Arxml 文件\n
        '''
        self.CanStack.StopCanStack()
        self.CanStack.ClearSubscribe()
        self.CanStack.ClearSend()
        self.CanStack.ClearLogger()
        self.CanParser.ClearSubscribe()
        self.CanParser.ClearCanDb()
        self.SomeipStack.Reset()



import json
import typing

import grpc

from . import ZoneSenderData
from .ObjIo import *

from .Protos import CanParserNode_pb2, CanParserNode_pb2_grpc


class CanParserNodeClient(object):
    def __init__(self) -> None:
        """ CanParserNode 的客户端
        """
        self._canDbParserStub = CanParserNode_pb2_grpc.CanParserNodeStub(
            channel=grpc.insecure_channel(
                target='{0}:{1}'
                    .format(
                        ZoneSenderData.CAN_PARSERNODE_NODE_IP, 
                        ZoneSenderData.CAN_PARSERNODE_NODE_PORT),
                options = ZoneSenderData.GRPC_OPTIONS
            )
        )

    def AddCanDbFile(self, db_path: 'str') -> 'int':
        ''' 添加一个DB文件

        :param db_path: str CAN DB 的文件路径
        :return: int

            - 0: 成功

            - 1: 解析失败

            - 2: 文件类型不能识别

            - 1000: error
        '''
        try:
            res_ = self._canDbParserStub.AddDbFile(
                CanParserNode_pb2.db_path(
                    db_path = db_path,
                )
            )
            print('AddCanDbFile result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def SetDBConfig(self, config_d: 'dict') -> 'int':
        ''' 设置 channel 与 CAN DB 的 Mapping

        :param config_d: dict 例如

            {0: 'PTCANFD',

            1: 'BOCAN'}

        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            l_ = list()
            for channel_, cluster_name_ in config_d.items():
                l_.append(CanParserNode_pb2.db_config_pair(
                    channel = channel_,
                    db_name = cluster_name_,
                ))
            res_ = self._canDbParserStub.SetConfig(
                CanParserNode_pb2.db_configs(
                    configs = l_
                )
            )
            print('SetCanDbConfig result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def ClearCanDb(self) -> 'int':
        ''' 清除所有的CANDB配置

        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            res_ = self._canDbParserStub.Clear(
                CanParserNode_pb2.Common__pb2.empty()
            )
            print('清除所有 CANDB 配置 result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def ClearSubscribe(self) -> 'int':
        ''' 清除 CanparserNode 的订阅客户端

        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            res_ = self._canDbParserStub.ClearSubscribe(
                CanParserNode_pb2.Common__pb2.empty()
            )
            print('清除所有 CanParser 的订阅 result: {0}, reason: {0}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def GetMqttTopicTreeJson(self, return_d: 'dict') -> 'int':
        ''' 获取所有的 Mqtt Tree dict 对象
        如果获取成功，将会在 return_d 中填充 TopicTree dict

        :param return_d: 要返回的 Mqtt Topic Tree 字典
        :return: int

            - 0: 获取成功

            - 1000: error
        '''
        try:
            res_ = self._canDbParserStub.GetMqttTopicTreeJson(
                CanParserNode_pb2.Common__pb2.empty()
            )
            if (not res_.result.result == 0):
                return res_.result.result
            parser_json_str_ = res_.str_json
            return_d.clear()
            return_d.update(json.loads(parser_json_str_))
            return res_.result.result
        except Exception as e_:
            print(e_)
            return 1000

    def CanEncode(self, can_obj) -> 'int':
        ''' 对 CAN 对象进行编码
        
        :param can_obj: 要编码的 CAN Obj 可能是 ZoneSender.ObjIo.CanISignalPdu
        :return: int\n
            - 0              编码成功\n
            - \-1             类型不对\n
            - 1              找不到对应的通道\n
            - 2              找不到对应的 PDU 名字\n
            - 3              找不到 PDU context 中指定的信号名\n
            - 1000           Error\n
        '''
        try:
            if (isinstance(can_obj, CanISignalIPdu)):
                res_ = self._canDbParserStub.EncodePdu(
                    CanParserNode_pb2.i_signal_i_pdu_obj(
                        channel=can_obj.channel,
                        pdu_name=can_obj.name,
                        pdu_context=json.dumps(can_obj.context)
                    )
                )
                result_ = res_.result.result
                if (result_ == 0):
                    can_obj.data = res_.data
                else:
                    print(res_.result.reason)
                return result_
            else:
                return -1
        except Exception as e_:
            print(e_)
            return 1000


from .CanFrame import CanFrame
from .CanMessage import CanMessage
from .CanSignal import CanSignal
from .CanISignalPdu import CanISignalIPdu
from .CanStackConfig import CanStackConfig
from .SomeipPackage import SomeipPackage


from copy import copy
import typing
import copy
from .ZoneSenderObject import ZoneSenderObject


class CanFrame(ZoneSenderObject):
    ''' CanFrame 对象，用来描述一个 CanFrame
    '''
    def __init__(
        self,
        name: 'str',
        channel: 'int',
        context: 'dict' = dict(),
        ) -> None:
        ...
        # #####################################
        super().__init__()
        self.name = name
        self.channel = channel
        self.context = copy.deepcopy(context)
        self.dlc = 0
        self.id = 0
        self.data = []

    def __str__(self) -> str:
        # return super().__str__()
        return 'name: {0} data: {2}'.format(self.name, self.data)


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import CanStackNode_pb2 as CanStackNode__pb2
from . import Common_pb2 as Common__pb2


class CanStackNodeStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetVersion = channel.unary_unary(
                '/CanStackNode.CanStackNode/GetVersion',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.version.FromString,
                )
        self.SetConfigs = channel.unary_unary(
                '/CanStackNode.CanStackNode/SetConfigs',
                request_serializer=CanStackNode__pb2.can_channel_configs.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.StartCanStack = channel.unary_unary(
                '/CanStackNode.CanStackNode/StartCanStack',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.StopCanStack = channel.unary_unary(
                '/CanStackNode.CanStackNode/StopCanStack',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.ClearSend = channel.unary_unary(
                '/CanStackNode.CanStackNode/ClearSend',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.ClearSubscribe = channel.unary_unary(
                '/CanStackNode.CanStackNode/ClearSubscribe',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.StartLog = channel.unary_unary(
                '/CanStackNode.CanStackNode/StartLog',
                request_serializer=CanStackNode__pb2.log_start_request.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.StopLog = channel.unary_unary(
                '/CanStackNode.CanStackNode/StopLog',
                request_serializer=CanStackNode__pb2.log_stop_request.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.ClearLogger = channel.unary_unary(
                '/CanStackNode.CanStackNode/ClearLogger',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )


class CanStackNodeServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetVersion(self, request, context):
        """获取版本
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetConfigs(self, request, context):
        """配置CAN协议栈
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartCanStack(self, request, context):
        """启动CAN协议栈
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopCanStack(self, request, context):
        """停止CAN协议栈
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClearSend(self, request, context):
        """清空定时CAN Message 发送列表
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClearSubscribe(self, request, context):
        """// 设置订阅信息
        rpc SubscribeCanMessage(subscribe_info) returns (Common.empty) {}

        // 取消订阅信息
        rpc UnSubscribeCanMessage(subscribe_info) returns (Common.empty) {}

        清除所有订阅
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartLog(self, request, context):
        """
        创建一个记录任务，任务执行结果：
        0: 执行成功
        1: 未知的记录类型
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopLog(self, request, context):
        """
        停止一个记录任务, 请求结果:
        1: 取消记录成功
        0: 取消记录失败
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClearLogger(self, request, context):
        """清除所有的 Logger
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_CanStackNodeServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetVersion': grpc.unary_unary_rpc_method_handler(
                    servicer.GetVersion,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.version.SerializeToString,
            ),
            'SetConfigs': grpc.unary_unary_rpc_method_handler(
                    servicer.SetConfigs,
                    request_deserializer=CanStackNode__pb2.can_channel_configs.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'StartCanStack': grpc.unary_unary_rpc_method_handler(
                    servicer.StartCanStack,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'StopCanStack': grpc.unary_unary_rpc_method_handler(
                    servicer.StopCanStack,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'ClearSend': grpc.unary_unary_rpc_method_handler(
                    servicer.ClearSend,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'ClearSubscribe': grpc.unary_unary_rpc_method_handler(
                    servicer.ClearSubscribe,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'StartLog': grpc.unary_unary_rpc_method_handler(
                    servicer.StartLog,
                    request_deserializer=CanStackNode__pb2.log_start_request.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'StopLog': grpc.unary_unary_rpc_method_handler(
                    servicer.StopLog,
                    request_deserializer=CanStackNode__pb2.log_stop_request.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'ClearLogger': grpc.unary_unary_rpc_method_handler(
                    servicer.ClearLogger,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'CanStackNode.CanStackNode', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class CanStackNode(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetVersion(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanStackNode.CanStackNode/GetVersion',
            Common__pb2.empty.SerializeToString,
            Common__pb2.version.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetConfigs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanStackNode.CanStackNode/SetConfigs',
            CanStackNode__pb2.can_channel_configs.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartCanStack(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanStackNode.CanStackNode/StartCanStack',
            Common__pb2.empty.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopCanStack(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanStackNode.CanStackNode/StopCanStack',
            Common__pb2.empty.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClearSend(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanStackNode.CanStackNode/ClearSend',
            Common__pb2.empty.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClearSubscribe(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanStackNode.CanStackNode/ClearSubscribe',
            Common__pb2.empty.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartLog(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanStackNode.CanStackNode/StartLog',
            CanStackNode__pb2.log_start_request.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopLog(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanStackNode.CanStackNode/StopLog',
            CanStackNode__pb2.log_stop_request.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClearLogger(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanStackNode.CanStackNode/ClearLogger',
            Common__pb2.empty.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from .TestAssist import TestAssist
from .ZoneSenderStone import ZoneSenderStone


class ZoneSenderApp(ZoneSenderStone):
    '''
    ZoneSender 的职能类，可以用来实现对硬件以及各种协议栈的完全控制\n
    该类继承于 ZoneSender.ZoneSenderFramework\n
    该类有几个事件触发型的函数，用户如果需要逻辑仿真的功能。\n
    必须重写下面的几个函数来实现自己的功能:\n
        - OnCanFrame\n
        - OnCanPdu\n
        - OnCanSignal\n
        - OnCanMessage\n
        - OnSomeipPackage\n
    该对象有几个成员变量，可以查看相应的对象介绍页面来了解函数列表:\n
        - CanStack: ZoneSender.CanStackNodeClient\n
        - CanParser: ZoneSender.CanParserNodeClient\n
        - SomeipStack: ZoneSender.SomeipNodeClient\n
    通过调用几个成员变量的函数实现各种控制功能\n
    '''
    def __init__(self) -> None:
        super().__init__()
        self.TA = TestAssist()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Common.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x43ommon.proto\x12\x06\x43ommon\"(\n\x06result\x12\x0e\n\x06result\x18\x01 \x01(\x05\x12\x0e\n\x06reason\x18\x02 \x01(\t\"\x07\n\x05\x65mpty\"0\n\x07version\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x14\n\x0crelease_date\x18\x02 \x01(\t\"*\n\x08net_info\x12\x0f\n\x07ip_addr\x18\x01 \x01(\t\x12\r\n\x05iface\x18\x02 \x01(\t\"\x19\n\tfile_path\x12\x0c\n\x04path\x18\x01 \x01(\tb\x06proto3')



_RESULT = DESCRIPTOR.message_types_by_name['result']
_EMPTY = DESCRIPTOR.message_types_by_name['empty']
_VERSION = DESCRIPTOR.message_types_by_name['version']
_NET_INFO = DESCRIPTOR.message_types_by_name['net_info']
_FILE_PATH = DESCRIPTOR.message_types_by_name['file_path']
result = _reflection.GeneratedProtocolMessageType('result', (_message.Message,), {
  'DESCRIPTOR' : _RESULT,
  '__module__' : 'Common_pb2'
  # @@protoc_insertion_point(class_scope:Common.result)
  })
_sym_db.RegisterMessage(result)

empty = _reflection.GeneratedProtocolMessageType('empty', (_message.Message,), {
  'DESCRIPTOR' : _EMPTY,
  '__module__' : 'Common_pb2'
  # @@protoc_insertion_point(class_scope:Common.empty)
  })
_sym_db.RegisterMessage(empty)

version = _reflection.GeneratedProtocolMessageType('version', (_message.Message,), {
  'DESCRIPTOR' : _VERSION,
  '__module__' : 'Common_pb2'
  # @@protoc_insertion_point(class_scope:Common.version)
  })
_sym_db.RegisterMessage(version)

net_info = _reflection.GeneratedProtocolMessageType('net_info', (_message.Message,), {
  'DESCRIPTOR' : _NET_INFO,
  '__module__' : 'Common_pb2'
  # @@protoc_insertion_point(class_scope:Common.net_info)
  })
_sym_db.RegisterMessage(net_info)

file_path = _reflection.GeneratedProtocolMessageType('file_path', (_message.Message,), {
  'DESCRIPTOR' : _FILE_PATH,
  '__module__' : 'Common_pb2'
  # @@protoc_insertion_point(class_scope:Common.file_path)
  })
_sym_db.RegisterMessage(file_path)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _RESULT._serialized_start=24
  _RESULT._serialized_end=64
  _EMPTY._serialized_start=66
  _EMPTY._serialized_end=73
  _VERSION._serialized_start=75
  _VERSION._serialized_end=123
  _NET_INFO._serialized_start=125
  _NET_INFO._serialized_end=167
  _FILE_PATH._serialized_start=169
  _FILE_PATH._serialized_end=194
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: CanParserNode.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import Common_pb2 as Common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13\x43\x61nParserNode.proto\x12\rCanParserNode\x1a\x0c\x43ommon.proto\"/\n\x0esubscribe_info\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\r\x12\x0c\n\x04name\x18\x02 \x01(\t\"\x1a\n\x07\x64\x62_path\x12\x0f\n\x07\x64\x62_path\x18\x01 \x01(\t\"<\n\ndb_configs\x12.\n\x07\x63onfigs\x18\x01 \x03(\x0b\x32\x1d.CanParserNode.db_config_pair\"2\n\x0e\x64\x62_config_pair\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\r\x12\x0f\n\x07\x64\x62_name\x18\x02 \x01(\t\"C\n\x0fmqtt_topic_tree\x12\x1e\n\x06result\x18\x01 \x01(\x0b\x32\x0e.Common.result\x12\x10\n\x08str_json\x18\x02 \x01(\t\"L\n\x12i_signal_i_pdu_obj\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\r\x12\x10\n\x08pdu_name\x18\x02 \x01(\t\x12\x13\n\x0bpdu_context\x18\x03 \x01(\t\"U\n\x15i_signal_i_pdu_encode\x12\x1e\n\x06result\x18\x01 \x01(\x0b\x32\x0e.Common.result\x12\x0e\n\x06length\x18\x02 \x01(\r\x12\x0c\n\x04\x64\x61ta\x18\x03 \x03(\r2\xae\x03\n\rCanParserNode\x12.\n\nGetVersion\x12\r.Common.empty\x1a\x0f.Common.version\"\x00\x12\x35\n\tAddDbFile\x12\x16.CanParserNode.db_path\x1a\x0e.Common.result\"\x00\x12\x38\n\tSetConfig\x12\x19.CanParserNode.db_configs\x1a\x0e.Common.result\"\x00\x12(\n\x05\x43lear\x12\r.Common.empty\x1a\x0e.Common.result\"\x00\x12\x31\n\x0e\x43learSubscribe\x12\r.Common.empty\x1a\x0e.Common.result\"\x00\x12G\n\x14GetMqttTopicTreeJson\x12\r.Common.empty\x1a\x1e.CanParserNode.mqtt_topic_tree\"\x00\x12V\n\tEncodePdu\x12!.CanParserNode.i_signal_i_pdu_obj\x1a$.CanParserNode.i_signal_i_pdu_encode\"\x00\x62\x06proto3')



_SUBSCRIBE_INFO = DESCRIPTOR.message_types_by_name['subscribe_info']
_DB_PATH = DESCRIPTOR.message_types_by_name['db_path']
_DB_CONFIGS = DESCRIPTOR.message_types_by_name['db_configs']
_DB_CONFIG_PAIR = DESCRIPTOR.message_types_by_name['db_config_pair']
_MQTT_TOPIC_TREE = DESCRIPTOR.message_types_by_name['mqtt_topic_tree']
_I_SIGNAL_I_PDU_OBJ = DESCRIPTOR.message_types_by_name['i_signal_i_pdu_obj']
_I_SIGNAL_I_PDU_ENCODE = DESCRIPTOR.message_types_by_name['i_signal_i_pdu_encode']
subscribe_info = _reflection.GeneratedProtocolMessageType('subscribe_info', (_message.Message,), {
  'DESCRIPTOR' : _SUBSCRIBE_INFO,
  '__module__' : 'CanParserNode_pb2'
  # @@protoc_insertion_point(class_scope:CanParserNode.subscribe_info)
  })
_sym_db.RegisterMessage(subscribe_info)

db_path = _reflection.GeneratedProtocolMessageType('db_path', (_message.Message,), {
  'DESCRIPTOR' : _DB_PATH,
  '__module__' : 'CanParserNode_pb2'
  # @@protoc_insertion_point(class_scope:CanParserNode.db_path)
  })
_sym_db.RegisterMessage(db_path)

db_configs = _reflection.GeneratedProtocolMessageType('db_configs', (_message.Message,), {
  'DESCRIPTOR' : _DB_CONFIGS,
  '__module__' : 'CanParserNode_pb2'
  # @@protoc_insertion_point(class_scope:CanParserNode.db_configs)
  })
_sym_db.RegisterMessage(db_configs)

db_config_pair = _reflection.GeneratedProtocolMessageType('db_config_pair', (_message.Message,), {
  'DESCRIPTOR' : _DB_CONFIG_PAIR,
  '__module__' : 'CanParserNode_pb2'
  # @@protoc_insertion_point(class_scope:CanParserNode.db_config_pair)
  })
_sym_db.RegisterMessage(db_config_pair)

mqtt_topic_tree = _reflection.GeneratedProtocolMessageType('mqtt_topic_tree', (_message.Message,), {
  'DESCRIPTOR' : _MQTT_TOPIC_TREE,
  '__module__' : 'CanParserNode_pb2'
  # @@protoc_insertion_point(class_scope:CanParserNode.mqtt_topic_tree)
  })
_sym_db.RegisterMessage(mqtt_topic_tree)

i_signal_i_pdu_obj = _reflection.GeneratedProtocolMessageType('i_signal_i_pdu_obj', (_message.Message,), {
  'DESCRIPTOR' : _I_SIGNAL_I_PDU_OBJ,
  '__module__' : 'CanParserNode_pb2'
  # @@protoc_insertion_point(class_scope:CanParserNode.i_signal_i_pdu_obj)
  })
_sym_db.RegisterMessage(i_signal_i_pdu_obj)

i_signal_i_pdu_encode = _reflection.GeneratedProtocolMessageType('i_signal_i_pdu_encode', (_message.Message,), {
  'DESCRIPTOR' : _I_SIGNAL_I_PDU_ENCODE,
  '__module__' : 'CanParserNode_pb2'
  # @@protoc_insertion_point(class_scope:CanParserNode.i_signal_i_pdu_encode)
  })
_sym_db.RegisterMessage(i_signal_i_pdu_encode)

_CANPARSERNODE = DESCRIPTOR.services_by_name['CanParserNode']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SUBSCRIBE_INFO._serialized_start=52
  _SUBSCRIBE_INFO._serialized_end=99
  _DB_PATH._serialized_start=101
  _DB_PATH._serialized_end=127
  _DB_CONFIGS._serialized_start=129
  _DB_CONFIGS._serialized_end=189
  _DB_CONFIG_PAIR._serialized_start=191
  _DB_CONFIG_PAIR._serialized_end=241
  _MQTT_TOPIC_TREE._serialized_start=243
  _MQTT_TOPIC_TREE._serialized_end=310
  _I_SIGNAL_I_PDU_OBJ._serialized_start=312
  _I_SIGNAL_I_PDU_OBJ._serialized_end=388
  _I_SIGNAL_I_PDU_ENCODE._serialized_start=390
  _I_SIGNAL_I_PDU_ENCODE._serialized_end=475
  _CANPARSERNODE._serialized_start=478
  _CANPARSERNODE._serialized_end=908
# @@protoc_insertion_point(module_scope)



import typing
import uuid
import copy

class ZoneSenderObject(object):
    ''' ZoneSender 中的基础数据对象
    '''
    def __init__(self) -> None:
        # self._triggerFuntion = None
        self.name = uuid.uuid4().hex
        self.context = dict()


import typing
import copy
from .ZoneSenderObject import ZoneSenderObject


class CanSignal(ZoneSenderObject):
    ''' CanSignal 对象，用来描述一个 CAN Signal
    '''
    def __init__(
        self,
        name: 'str',
        channel: 'int',
        data: 'float' = 0.0,
        mean: 'str' = '',
        unit: 'str' = '',
        context: 'dict' = dict(),
        ) -> None:
        ...
        # #####################################
        super().__init__()
        self.name = name
        self.data = data
        self.channel = channel
        self.mean = mean
        self.unit = unit
        self.context = copy.deepcopy(context)

    def __str__(self) -> str:
        # return super().__str__()
        return 'name: {0} channel: {1} value: {2} mean: {3}'.format(self.name, self.channel, self.data, self.mean)